#!/usr/bin/gnuplot

# Miroslav Broz (miroslav.broz@email.cz), Jul 26th 2002

set colors classic
set term x11

set tit ""
set xl "diameter {/Helvetica-Oblique D} / km"
set yl "number of asteroids {/Helvetica-Oblique N} (>{/Helvetica-Oblique D})" offset +0.5

set xr [0.1:600]
miny=0.8
maxy=1e6
set yr [miny:maxy]
set logscale xy
set mxtics 10
set ytics 10
set mytics 10
set key samplen 1.0

set xtics (\
  "0.1"  1e-1 0,\
  ""     2e-1 1,\
  ""     3e-1 1,\
  ""     4e-1 1,\
  ""     5e-1 1,\
  ""     6e-1 1,\
  ""     7e-1 1,\
  ""     8e-1 1,\
  ""     9e-1 1,\
  "1"    1e0  0,\
  ""     2e0  1,\
  ""     3e0  1,\
  ""     4e0  1,\
  ""     5e0  1,\
  ""     6e0  1,\
  ""     7e0  1,\
  ""     8e0  1,\
  ""     9e0  1,\
  "10"   1e1  0,\
  ""     2e1  1,\
  ""     3e1  1,\
  ""     4e1  1,\
  ""     5e1  1,\
  ""     6e1  1,\
  ""     7e1  1,\
  ""     8e1  1,\
  ""     9e1  1,\
  "100"  1e2  0,\
  ""     2e2  1,\
  ""     3e2  1,\
  ""     4e2  1,\
  "500"  5e2  0,\
  )
set ytics (\
  ""     8e-1 1,\
  ""     9e-1 1,\
  "1"    1e0  0,\
  ""     2e0  1,\
  ""     3e0  1,\
  ""     4e0  1,\
  ""     5e0  1,\
  ""     6e0  1,\
  ""     7e0  1,\
  ""     8e0  1,\
  ""     9e0  1,\
  "10"   1e1  0,\
  ""     2e1  1,\
  ""     3e1  1,\
  ""     4e1  1,\
  ""     5e1  1,\
  ""     6e1  1,\
  ""     7e1  1,\
  ""     8e1  1,\
  ""     9e1  1,\
  "100"  1e2  0,\
  ""     2e2  1,\
  ""     3e2  1,\
  ""     4e2  1,\
  ""     5e2  1,\
  ""     6e2  1,\
  ""     7e2  1,\
  ""     8e2  1,\
  ""     9e2  1,\
  "1000" 1e3  0,\
  ""     2e3  1,\
  ""     3e3  1,\
  ""     4e3  1,\
  ""     5e3  1,\
  ""     6e3  1,\
  ""     7e3  1,\
  ""     8e3  1,\
  ""     9e3  1,\
  "{/=14 10^4}" 1e4  0,\
  ""     2e4  1,\
  ""     3e4  1,\
  ""     4e4  1,\
  ""     5e4  1,\
  ""     6e4  1,\
  ""     7e4  1,\
  ""     8e4  1,\
  ""     9e4  1,\
  "{/=14 10^5}" 1e5  0,\
  ""     2e5  1,\
  ""     3e5  1,\
  ""     4e5  1,\
  ""     5e5  1,\
  ""     6e5  1,\
  ""     7e5  1,\
  ""     8e5  1,\
  ""     9e5  1,\
  "{/=14 10^6}" 1e6  0,\
  ""     2e6  1,\
  ""     3e6  1,\
  ""     4e6  1,\
  ""     5e6  1,\
  ""     6e6  1,\
  ""     7e6  1,\
  ""     8e6  1,\
  ""     9e6  1,\
  "{/=14 10^7}" 1e7  0,\
  ""     2e7  1,\
  ""     3e7  1,\
  ""     4e7  1,\
  ""     5e7  1,\
  ""     6e7  1,\
  ""     7e7  1,\
  ""     8e7  1,\
  ""     9e7  1,\
  "{/=14 10^8}" 1e8  0,\
  ""     2e8  1,\
  ""     3e8  1,\
  ""     4e8  1,\
  ""     5e8  1,\
  ""     6e8  1,\
  ""     7e8  1,\
  ""     8e8  1,\
  ""     9e8  1,\
  "{/=14 10^9}" 1e9  0,\
  ""     2e9  1,\
  ""     3e9  1,\
  ""     4e9  1,\
  ""     5e9  1,\
  ""     6e9  1,\
  ""     7e9  1,\
  ""     8e9  1,\
  ""     9e9  1,\
  "{/=14 10^{10}}" 1e10 0,\
  ""     2e10 1,\
  ""     3e10 1,\
  ""     4e10 1,\
  ""     5e10 1,\
  ""     6e10 1,\
  ""     7e10 1,\
  ""     8e10 1,\
  ""     9e10 1,\
  "{/=14 10^{11}}" 1e11 0,\
  ""     2e11 1,\
  ""     3e11 1,\
  ""     4e11 1,\
  ""     5e11 1,\
  ""     6e11 1,\
  ""     7e11 1,\
  ""     8e11 1,\
  ""     9e11 1,\
  "{/=14 10^{12}}" 1e12 0,\
  )

set arrow from 1,graph 0 rto 0,graph 1 nohead lt 0

set lmargin 7.0
set rmargin 0.5


f_family7(x) = (10**3.96406343e+00) * x**(-1.53578421e+00)  # Koronis
f_family8(x) = (10**3.04947554e+00) * x**(-2.91181685e+00)  # Karin

p \
  f_family7(x) not lc '#dd00ff' dt 2,\
  f_family8(x) not lc 'green'   dt 2,\
  "../158_Koronis/size_distribution/family.dat_hc" w l lc '#dd00ff' lw 3 t "Koronis (L/H)",\
  "../832_Karin/size_distribution/family.dat_hc"   w l lc '#ffaaff' lw 3 t "Karin (L/H)"

pa -1

set term postscript eps enh color solid "Helvetica" 18
set out "Koronis_Karin.eps"
set size 0.625,0.9
rep

system("eps2png Koronis_Karin.eps")

q


